##############################################################################
#
# Copyright (c) 2003-2005 struktur AG and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: queue.py 14142 2005-11-21 17:09:03Z dreamcatcher $
"""

import thread
from sets import Set
from zLOG import LOG, DEBUG
from Shared.DC.ZRDB.TM import TM
from Products.CMFCore.utils import getToolByName
from Products.CMFSquidTool.utils import pruneUrl
from Products.CMFSquidTool.patch import wrap_method, call

def log(msg, level=DEBUG):
    LOG('CMFSquidTool', level, msg)

class Queue(TM):
    """ Main Purge Queue

    - Keeps a queue of urls to invalidate per thread.
    - Sends purge requests on transaction commit.
    """

    def __init__(self):
        self.__urls = {}

    def _tid(self):
        return thread.get_ident()

    def urls(self):
        tid = self._tid()
        return self.__urls.get(tid, ())

    def _reset(self):
        tid = self._tid()
        self.__urls[tid] = Set()

    def _begin(self):
        assert not self.urls(), 'Unprocessed urls found in queue'

    def _abort(self):
        # Throw away all catched urls for this thread
        self._reset()

    def _finish(self):
        # Process any pending url invalidations. This should *never*
        # fail.
        for url in self.urls():
            (status, xcache, xerror) = pruneUrl(url, purge_type='PURGE')
            log('PURGE %s: %s %s %s' % (url, status, xcache, xerror))
        # Empty urls queue for this thread
        self._reset()

    def append(self, url):
        self._register()
        tid = self._tid()
        urls = self.__urls.setdefault(tid, Set())
        urls.add(url)

    def queue(self, ob):
        st = getToolByName(ob, 'portal_squid', None)
        if st is None:
            return
        ob_urls = st.getUrlsToPurge(ob)
        purge_urls = st.computePurgeUrls(ob_urls)
        for ob_url in purge_urls:
            self.append(ob_url)

queue = Queue()

def reindexObject(self, idxs=[]):
    """Reindex Object

    Queue PURGE url and call original method.
    """
    queue.queue(self)
    return call(self, 'reindexObject', idxs=idxs)

from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
wrap_method(CMFCatalogAware, 'reindexObject', reindexObject)

try:
    from Products.Archetypes.CatalogMultiplex import CatalogMultiplex
    wrap_method(CatalogMultiplex, 'reindexObject', reindexObject)
except ImportError:
    log('Archetypes not found. Not patching CatalogMultiplex.')
